/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create;

import java.util.Collection;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.Opt4JDseTaskProvider;
import org.fortiss.af3.exploration.alg.dse.module.CreatorModule;
import org.opt4j.core.Genotype;
import org.opt4j.core.genotype.CompositeGenotype;
import org.opt4j.core.problem.Creator;
import org.opt4j.core.start.Opt4JTask;

import com.google.inject.AbstractModule;
import com.google.inject.Inject;
import com.google.inject.Injector;

/**
 * Dummy creator to allow registering the Guice Decoder.
 * 
 * @author diewald
 */
public class GuiceCreator extends AbstractModule implements
		Creator<CompositeGenotype<Class<? extends Genotype>, ? extends Genotype>> {

	/** Set of {@link Genotype}-Types for which instances shall be created. */
	private Set<Class<? extends Genotype>> requiredGenotypes;

	/** Set of {@link CreatorModule}s provided by the enabled exploration features. */
	private Collection<CreatorModule<?>> enabledCreators;

	/** {@link Opt4JTask} provider used to obtain the opt4j injector. */
	@Inject
	protected Opt4JDseTaskProvider opt4JProvider;

	/** Child {@link Injector} for creating the required {@link Genotype}s in the creation phase. */
	protected Injector createInjector;

	/** Constructor. */
	public GuiceCreator(Set<Class<? extends Genotype>> requiredGenotypes,
			Collection<CreatorModule<?>> enabledCreators) {
		this.requiredGenotypes = requiredGenotypes;
		this.enabledCreators = enabledCreators;
	}

	/** {@inheritDoc} */
	@Override
	public CompositeGenotype<Class<? extends Genotype>, ? extends Genotype> create() {
		createInjector = opt4JProvider.get().get().createChildInjector(this);

		CompositeGenotype<Class<? extends Genotype>, Genotype> compGenotype =
				new CompositeGenotype<>();
		for(Class<? extends Genotype> requiredGenotype : requiredGenotypes) {
			compGenotype.put(requiredGenotype, createInjector.getInstance(requiredGenotype));
		}

		return compGenotype;
	}

	/** {@inheritDoc} */
	@Override
	protected void configure() {
		// bindScope(ThreadScoped.class, new ThreadScope());

		for(CreatorModule<?> module : enabledCreators) {
			install(module);
		}
	}
}
