/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.PluginDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.opt4j.core.Genotype;

import com.google.inject.Provides;
import com.google.inject.Singleton;

/**
 * Provides a {@link AbstractTaskMappingEncoding} to the set of {@link Phenotype} types.
 * This {@link ComposableDecoder} is an identity operation.
 * 
 * @author diewald
 */
// TODO: We actually need an additional AbstractTaskMappingDecoder, as this one is not an identity
// decoder anymore. We'd need a clearer concept when to use which operators anyways (Dependent on
// the selected features of the DSE).
@Singleton
public class AbstractTaskMappingIdentityDecoder extends
		TaskMappingDecoderBase<AbstractTaskMappingEntry, AbstractTaskMappingEncoding> {

	/**
	 * Decodes/Updates the given {@link AbstractTaskMappingEncoding} with the number of additional
	 * resources of an update {@link PartitionMappingEncoding}.
	 */
	@Provides
	@PluginDecoder
	public AbstractTaskMappingEncoding decode(@Genotyped AbstractTaskMappingEncoding atmEnc,
			@Genotyped PartitionMappingEncoding partEnc) {
		return alignWithResourceModifications(atmEnc, partEnc);
	}

	/** {@inheritDoc} */
	@Override
	public AbstractTaskMappingEncoding decode(Genotype genotype) {
		return null;
	}
}
